--[[
	Enumerate
	v1.01 3rd September 2017

	Copyright (C) by Taraezor / Chris Birch
]]
--------------------------------------------------------------------------------------------------------------------------------------------

local addonName, L = ...

local format = string.format
local gmatch = string.gmatch
local lower = string.lower
local sub = string.sub
local tostring = tostring

local EnumerateFrames = EnumerateFrames
local GetAddOnMetadata = GetAddOnMetadata
local MouseIsOver = MouseIsOver

--------------------------------------------------------------------------------------------------------------------------------------------
-- Print Colours - Chocolate theme
--------------------------------------------------------------------------------------------------------------------------------------------

local pc_colour_Prefix		= "\124cFFD2691E"	-- X11Chocolate
local pc_colour_Highlight	= "\124cFFF4A460"	-- X11SandyBrown
local pc_colour_PlainText	= "\124cFFDEB887"	-- X11BurlyWood

local function printPC( message )
	DEFAULT_CHAT_FRAME:AddMessage( pc_colour_Prefix.. addonName.. ": ".. pc_colour_PlainText.. message.. "\124r" )
end

--------------------------------------------------------------------------------------------------------------------------------------------
--
--		Event Handler
--		=============
--
--------------------------------------------------------------------------------------------------------------------------------------------

local function EventHandler( self, event, addonNameEvent)

	if ( event == "ADDON_LOADED" ) and ( addonNameEvent == addonName ) then
		if not EnumerateDB then EnumerateDB = {} end
		local matchParent = EnumerateDB.matchParent
		local showParent = EnumerateDB.showParent or "h"
		EnumerateDB = {}
		EnumerateDB.matchParent = matchParent
		EnumerateDB.showParent = showParent
	end
end

local eventFrame = CreateFrame( "Frame" )
eventFrame:RegisterEvent( "ADDON_LOADED" )
eventFrame:SetScript( "OnEvent", EventHandler )

--------------------------------------------------------------------------------------------------------------------------------------------

local function ListRegions(...)
	for i = 1, select("#", ...) do
        	local region = select(i, ...)
		if region then
			printPC( "i="..i.." Type=".. region:GetObjectType().. " Name=".. tostring( region:GetName()) )
		end
	end
end

local function EstablishCredentials( frame )

	local paternity, parent = true;

	if EnumerateDB.matchParent or ( EnumerateDB.showParent == "s" ) then
		local parentFrame = frame:GetParent()
		if EnumerateDB.matchParent then
			paternity = parentFrame and ( parentFrame:GetName() and (parentFrame:GetName() == EnumerateDB.matchParent) or false ) or false
		end
		if EnumerateDB.showParent == "s" then
			parent = pc_colour_Highlight.. " Parent:".. pc_colour_PlainText.. 
				(parentFrame and (parentFrame:GetName() or "Unnamed") or "nil")
		end
	end
	return paternity, parent
end

--------------------------------------------------------------------------------------------------------------------------------------------
--
--		Slash Commands
--		==============
--
--------------------------------------------------------------------------------------------------------------------------------------------

SLASH_Enumerate1 = "/enu"

SlashCmdList[ "Enumerate" ] = function( options )

	local secondParm, options1
	for v in gmatch( options, "(%S+)" ) do
		if not options1 then
			options1 = sub( lower(v),1,1 )
		else
			secondParm = v
			break
		end
	end

	if ( options1 == "?" ) or not options1 then
		printPC( pc_colour_Highlight .. "Macro Commands:\n"..
			pc_colour_Highlight.. "a".. pc_colour_PlainText.. " = Enumerate all visible frames anywhere\n"..
			pc_colour_Highlight.. "n".. pc_colour_PlainText.. " = Enumerate all visible named frames anywhere\n"..
			pc_colour_Highlight.. "m".. pc_colour_PlainText.. " = Enumerate all visible frames under the mouse cursor\n"..
			pc_colour_Highlight.. "r".. pc_colour_PlainText.. " = Enumerate all visible frames & regions under the mouse cursor\n"..
			pc_colour_Highlight.. "p [name] / ['s'/'h']".. pc_colour_PlainText.. " = Match parent name. 's'/'h' as show/hide switches (Nil to ignore)\n"..
			pc_colour_Highlight.. "v".. pc_colour_PlainText.. " = Show the version number\n")

	elseif ( options1 == "a" ) then
		local frame = EnumerateFrames()
		while frame do
			if ( frame:IsVisible() ) then
				local paternity, parent = EstablishCredentials( frame )
				if paternity == true then
					printPC( (frame:GetName() or string.format( "[Unnamed Frame: %s]", tostring( frame ) )).. (parent or "") )
				end
			end
			frame = EnumerateFrames(frame)
		end

	elseif ( options1 == "n" ) then
		local frame = EnumerateFrames()
		while frame do
			if frame:IsVisible() and frame:GetName() then
				local paternity, parent = EstablishCredentials( frame )
				if paternity == true then

local name = frame:GetName()
if sub( name, 1, 9 ) == "ChatFrame" or sub( name, 1, 6 ) == "Action" or sub( name, 1, 5 ) == "Multi" or sub( name, 1, 3 ) == "PVP" or sub( name, 1, 7 ) == "Minimap" or 
			sub( name, 1, 8 ) == "MainMenu" or sub( name, 1, 7 ) == "Minimap" or sub( name, 1, 7 ) == "Auction" or sub( name, 1, 9 ) == "Objective" then
else

					printPC( frame:GetName().. (parent or "") )
end
				end
			end
			frame = EnumerateFrames(frame)
		end

	elseif ( options1 == "m" ) then
		local frame = EnumerateFrames()
		while frame do
			if ( frame:IsVisible() and MouseIsOver( frame) ) then
				local paternity, parent = EstablishCredentials( frame )
				if paternity == true then
					printPC( (frame:GetName() or string.format( "[Unnamed Frame: %s]", tostring( frame ) )).. (parent or "") )
				end
			end
			frame = EnumerateFrames(frame)
		end

	elseif ( options1 == "r" ) then
		local frame = EnumerateFrames()
		while frame do
			if ( frame:IsVisible() and MouseIsOver( frame) ) then
				local paternity, parent = EstablishCredentials( frame )
				if paternity == true then
					printPC( (frame:GetName() or string.format( "[Unnamed Frame: %s]", tostring( frame ) )).. (parent or "") )
					ListRegions( frame:GetRegions() )
				end
			end
			frame = EnumerateFrames(frame)
		end

	elseif ( options1 == "p" ) then
		if secondParm and ( secondParm == "s" or secondParm == "h" ) then
			EnumerateDB.showParent = secondParm
		else
			EnumerateDB.matchParent = secondParm
		end
		printPC( "Parent: ".. pc_colour_Highlight.. ( EnumerateDB.matchParent or "nil" ).. pc_colour_PlainText..
			( EnumerateDB.showParent == "s" and " (Show)" or " (Hide)" ) )

	elseif ( options1 == "v" ) then
		local version = GetAddOnMetadata( addonName, "Version" )
		printPC( "Version: ".. pc_colour_Highlight.. version )
	end
end